@@ -6,10 +6,10 @@
 import asyncio
 import inspect
 import functools
-from typing import List, Callable  # noqa
+from typing import List, Callable, Tuple, Any, Dict
 import testslide
 from testslide.strict_mock import StrictMock
-from testslide.lib import _wrap_signature_validation, _validate_return_type
+from testslide.lib import _wrap_signature_and_type_validation, _validate_return_type
 from .patch import _patch, _is_instance_method
 from .lib import _bail_if_private
 
@@ -47,8 +47,8 @@ def _default_register_assertion(assertion):
 
 register_assertion = _default_register_assertion
 _call_order_assertion_registered = False
-_received_ordered_calls = []
-_expected_ordered_calls = []
+_received_ordered_calls: List[Tuple[Any, str, "_BaseRunner"]] = []
+_expected_ordered_calls: List[Tuple[Any, str, "_BaseRunner"]] = []
 
 
 def unpatch_all_callable_mocks():
@@ -448,11 +448,12 @@ class _AsyncCallOriginalRunner(_AsyncRunner):
 
 
 class _CallableMock(object):
-    def __init__(self, target, method, is_async=False):
+    def __init__(self, target, method, is_async=False, type_validation=True):
         self.target = target
         self.method = method
         self.runners = []
         self.is_async = is_async
+        self.type_validation = type_validation
 
     def _get_runner(self, *args, **kwargs):
         for runner in self.runners:
@@ -461,7 +462,11 @@ class _CallableMock(object):
         return None
 
     def _validate_return_type(self, runner, value):
-        if runner.TYPE_VALIDATION and runner.original_callable is not None:
+        if (
+            self.type_validation
+            and runner.TYPE_VALIDATION
+            and runner.original_callable is not None
+        ):
             _validate_return_type(runner.original_callable, value)
 
     def __call__(self, *args, **kwargs):
@@ -521,7 +526,7 @@ class _CallableMock(object):
 
 class _MockCallableDSL(object):
 
-    CALLABLE_MOCKS = {}  # NOQA T484
+    CALLABLE_MOCKS: Dict[int, _CallableMock] = {}
 
     def _validate_patch(
         self,
@@ -621,7 +626,7 @@ class _MockCallableDSL(object):
             original_callable = getattr(self._target, self._method)
 
         if not isinstance(self._target, StrictMock):
-            new_value = _wrap_signature_validation(
+            new_value = _wrap_signature_and_type_validation(
                 new_value, self._target, self._method, self.type_validation
             )
 
@@ -638,7 +643,9 @@ class _MockCallableDSL(object):
         return original_callable, unpatcher
 
     def _get_callable_mock(self):
-        return _CallableMock(self._original_target, self._method)
+        return _CallableMock(
+            self._original_target, self._method, type_validation=self.type_validation
+        )
 
     def __init__(
         self,
@@ -955,7 +962,12 @@ class _MockAsyncCallableDSL(_MockCallableDSL):
         )
 
     def _get_callable_mock(self):
-        return _CallableMock(self._original_target, self._method, is_async=True)
+        return _CallableMock(
+            self._original_target,
+            self._method,
+            is_async=True,
+            type_validation=self.type_validation,
+        )
 
     def with_implementation(self, func):
         """
